import os

class ClassGenerator():
    '''
    ClassGenerator
    This class is used to generate a class file.
    
    Params:
        name: The name of the class.
        members: A list of members to be added to the class.
        methods: A list of methods to be added to the class.
        constructor: The name of the constructor.
        constructor_args: A list of arguments to be passed to the constructor.

    Returns:
        None
    '''
    name: str
    members: list()
    getters = list()
    setters = list()
    constructor: str
    constructor_args: list()

    def __init__(self, name: str, members: list(), constructor_args: list()):
        '''
        __init__
        Initializes the class generator.
        
        Params:
            name: The name of the class.
            members: A list of members to be added to the class.
            constructor: The name of the constructor.
            constructor_args: A list of arguments to be passed to the constructor.

        Returns:
            None
        '''
        self.name = name
        self.members = members
        self.constructor_args = constructor_args
        # Create the getters and setters
        for member in members:
            self.getters.append('get_' + member[0] + member[1:])
            self.setters.append('set_' + member[0] + member[1:])
        # Add the constructor

    def generate_class_docstring(self):
        '''
        Generate the class docstring.
        '''
        docstring = str()
        # Create the docstring
        docstring += ('\t\"\"\"\n')
        docstring += ('\tThis class is used to represent a ' + self.name + '.\n')
        # Write details of attributes
        docstring += ('\t\n')
        docstring += ('\tAttributes:\n')
        for member in self.members:
            docstring += ('\t\t' + member + ': The ' + member + ' of the ' + self.name + '.\n')
        docstring += ('\t\n')
        
        # Write details of methods
        docstring += ('\tMethods:\n')
        for getter in self.getters:
            docstring += ('\t\t' + getter + '(self): Gets the ' + getter[4:] + ' of the ' + self.name + '.\n')
        for setter in self.setters:
            docstring += ('\t\t' + setter + '(self, ' + setter[4:] + ')' +  ': Sets the ' + setter[4:] + ' of the ' + self.name + '.\n')        
        docstring += ('\t\"\"\"\n')
        docstring += ('\n')
        return docstring

    def generate_members(self):
        '''
        Generate the members docstring.
        '''
        docstring = str()
        # Create the members
        for member in self.members:
            docstring += ('\t_' + member + ': ' + 'None\n')
        docstring += "\t\n"
        return docstring

    def generate_constructor_docstring(self):
        '''
        Generate the constructor docstring.
        '''
        docstring = str()
        # Create the docstring
        docstring += ('\t\t\"\"\"\n')
        docstring += ('\t\tInitializes a ' + self.name + ' object.\n')
        docstring += ('\t\t\n')
        docstring += ('\t\tParams:\n')
        for arg in self.constructor_args:
            docstring += ('\t\t\t' + arg + ': The ' + arg + ' of the ' + self.name + '.\n')
        docstring += ('\t\t\"\"\"\n')
        return docstring

    def generate_constructor(self):
        # Write the constructor
        docstring = str()
        docstring += ('\tdef __init__(self')
        for arg in self.constructor_args:
            docstring += (', ' + arg)
        docstring += ('):\n')
        docstring += (self.generate_constructor_docstring())
        # Write the constructor body
        for member in self.members:
            docstring += ('\t\tself._' + member + ' = ' + member + '\n')
        docstring += ('\t\n')
        return docstring
    
    def generate_getter_docstring(self, member):
        '''
        Generate the getter methods' docstrings.
        '''
        docstring = str()
        # Create the docstring
        docstring += ('\t\t\"\"\"\n')
        docstring += ('\t\tGets the ' + member[4:] + ' of the ' + self.name + '.\n')
        docstring += ('\t\t\n')
        docstring += ('\t\tReturns:\n')
        docstring += ('\t\t\t' + member + ': The ' + member[4:] + ' of the ' + self.name + '.\n')
        docstring += ('\t\t\"\"\"\n')
        return docstring

    def generate_getter(self):
        # Write the getters and setters
        docstring = str()
        for getter in self.getters:
            docstring += ('\tdef ' + getter + '(self):\n')
            docstring += (self.generate_getter_docstring(getter))
            docstring += ('\t\treturn self.' + getter[3:] + '\n')
            docstring += ('\n')
        return docstring
    
    def generate_setter_docstring(self, member):
        '''
        Generate the setter methods' docstrings.
        '''
        docstring = str()
        # Create the docstring
        docstring += ('\t\t\"\"\"\n')
        docstring += ('\t\tSets the ' + member[4:] + ' of the ' + self.name + '.\n')
        docstring += ("\t\t\n")
        docstring += ('\t\tParams:\n')
        docstring += ('\t\t\t' + member[4:] + ': The ' + member[4:] + ' of the ' + self.name + '.\n')
        docstring += ('\t\t\"\"\"\n')
        return docstring

    def generate_setter(self):
        docstring = str()
        for setter in self.setters:
            docstring += ('\tdef ' + setter + '(self, ' + setter[4:] + '):\n')
            docstring += (self.generate_setter_docstring(setter))
            docstring += ('\t\tself.' + setter[3:] + ' = ' + setter[4:] + '\n')
            docstring += ('\n')
            continue
        return docstring

    def generate(self, path: str="./name.py"):
        '''
        generate
        Generates the class file along with docstrings.
        
        Params:
            path: The path to the class file.

        Returns:
            None
        '''
        # Name resolution of the file
        if path == "./name.py":
            path = "./" + self.name + ".py"
        # If file exists, delete the file
        if os.path.exists(path):
            os.remove(path)
        # Create the file
        file = open(path, 'w')
        # Write the class header
        file.write(f'#(class) {self.name}\n')
        file.write('class ' + self.name + ':\n')
        file.write(self.generate_class_docstring())                
        file.write(self.generate_members())
        file.write(self.generate_constructor())
        file.write(self.generate_getter())
        file.write(self.generate_setter())        
        # Close the file
        file.close()
        return None


from sys import modules
from .classgen import ClassGenerator


# export the ClassGenerator class
modules['__all__'] = ['ClassGenerator']
modules['__dict__'] = {
    'ClassGenerator': ClassGenerator
}
modules['name'] = 'ClassGenerator'
modules['author'] = 'Avanish Gupta'


from sys import modules
from .ClassGenerator import ClassGenerator

# Export the ClassGenerator class
modules['__all__'] = ['ClassGenerator']
modules['name'] = 'ClassGenerator'
modules['author'] = 'Avanish Gupta'


from .classgen import ClassGenerator
from argparse import ArgumentParser


# Command line arguments
parser = ArgumentParser(description='Generate a class file.')
parser.add_argument(
    'className', 
    type=str, 
    help='The name of the class.'
)
parser.add_argument(
    'members', 
    type=str, 
    help='The members of the class.'
)
args = parser.parse_args()
if type(args.className) == str:
    cg = ClassGenerator(
        args.className,
        args.members.split(','),
        args.members.split(',')
    )
    cg.generate()
    print(f'Class {args.className} generated successfully.')
else:
    print('Error: Please follow the convention of the command line arguments.')
    print('Example: python3 ClassGenerator Student marks,grade,address')


from setuptools import setup, find_packages

with open("README.md", "r") as file:
    long_description = file.read()

setup(
    name="classgen",                            # This is the name of the package
    version="0.0.8",                            # The initial release version
    author="Avanish Gupta",                     # Full name of the author
    description="Generate classes in Python using command line arguments",
    long_description=long_description,      # Long description read from the the readme file
    long_description_content_type="text/markdown",
    packages=find_packages(),    # List of all python modules to be installed
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],                                       # Information to filter the project on PyPi website
    python_requires='>=3.6',                 # Minimum version requirement of the package
    py_modules=["classgen"],                 # Name of the python package
    install_requires=[],                     # Install other dependencies if any
    keywords=['Class', 'generate', 'automation', 'command line arguments', 'cli'],
    # package_dir={'':'classgenerator'},     # Directory of the source code of the package
)